from typing import Dict, Set

from nicovideo_api_client.api.v2.sort import SnapshotSearchAPIV2Sort
from nicovideo_api_client.constants import FieldType


class SnapshotSearchAPIV2Fields:
    def __init__(self, query: Dict[str, str]):
        self._fields: Set[str] = set()
        self._query: Dict[str, str] = query

    def field(self, fields: Set[FieldType]) -> "SnapshotSearchAPIV2Sort":
        """
        レスポンスの返すフィールドタイプを指定する。

        :param fields: レスポンスの返すフィールドタイプ一覧
        :return: ソートタイプ指定オブジェクト
        """
        self._fields = sorted(list(map(lambda x: x.value, fields)))
        if len(self._fields) > 0:
            self._query["fields"] = ",".join(self._fields)
        return SnapshotSearchAPIV2Sort(self._query)



from enum import Enum
from typing import List, Dict, Union, Literal, TypeAlias
from datetime import datetime


class FieldType(Enum):
    """コンテンツのフィールド種別"""

    CONTENT_ID = "contentId"
    """コンテンツID
    https://nico.ms/ の後に連結することでコンテンツへのURLになります
    """

    TITLE = "title"
    """タイトル"""

    DESCRIPTION = "description"
    """コンテンツの説明文"""

    USER_ID = "userId"
    """ユーザー投稿動画の場合、投稿者のユーザーID"""

    CHANNEL_ID = "channelId"
    """	チャンネル動画の場合、チャンネルID"""

    VIEW_COUNTER = "viewCounter"
    """再生数"""

    MYLIST_COUNTER = "mylistCounter"
    """マイリスト数またはお気に入り数"""

    LIKE_COUNTER = "likeCounter"
    """いいね！数"""

    LENGTH_SECONDS = "lengthSeconds"
    """再生時間(秒)"""

    THUMBNAIL_URL = "thumbnailUrl"
    """サムネイルのURL"""

    START_TIME = "startTime"
    """コンテンツの投稿時間"""

    LAST_RES_BODY = "lastResBody"
    """最新のコメント"""

    COMMENT_COUNTER = "commentCounter"
    """コメント数"""

    LAST_COMMENT_TIME = "lastCommentTime"
    """最終コメント時間"""

    CATEGORY_TAGS = "categoryTags"
    """カテゴリタグ"""

    TAGS = "tags"
    """タグ(空白区切り)"""

    TAGS_EXACT = "tagsExact"
    """タグ完全一致(空白区切り)"""

    GENRE = "genre"
    """ジャンル"""

    GENRE_KEYWORD = "genre.keyword"
    """ジャンル完全一致"""


DEFAULT_RETRY = 3

END_POINT_URL_V2 = (
    "https://api.search.nicovideo.jp/api/v2/snapshot/video/contents/search"
)
END_POINT_URL_VIDEO = "https://www.nicovideo.jp/watch/"
END_POINT_URL_V2_VERSION = "https://api.search.nicovideo.jp/api/v2/snapshot/version"

sort_types = [
    FieldType.VIEW_COUNTER,
    FieldType.MYLIST_COUNTER,
    FieldType.LENGTH_SECONDS,
    FieldType.START_TIME,
    FieldType.COMMENT_COUNTER,
    FieldType.LAST_COMMENT_TIME,
]

target_types = [
    FieldType.CONTENT_ID,
    FieldType.TITLE,
    FieldType.DESCRIPTION,
    FieldType.TAGS,
    FieldType.LAST_RES_BODY,
    FieldType.CATEGORY_TAGS,
    FieldType.TAGS,
    FieldType.TAGS_EXACT,
    FieldType.GENRE,
    FieldType.GENRE_KEYWORD,
]


MatchValue: TypeAlias = Union[List[str], List[int], List[datetime]]
RangeLiteral: TypeAlias = Literal["gt", "gte", "lt", "lte"]
RangeValue: TypeAlias = Union[
    Dict[RangeLiteral, str], Dict[RangeLiteral, int], Dict[RangeLiteral, datetime]
]
MatchDict: TypeAlias = Dict[FieldType, MatchValue]
RangeDict: TypeAlias = Dict[FieldType, RangeValue]
CombinedDict: TypeAlias = Dict[FieldType, Union[MatchValue, RangeValue]]

JSON_LITERAL: TypeAlias = str | int | float | bool | None
JSON: TypeAlias = JSON_LITERAL | Dict[str, "JSON"] | List["JSON"]



import math
import statistics
from json import JSONEncoder
from typing import Any, Dict, List, Optional, Union

import requests

from nicovideo_api_client.constants import FieldType


class SnapshotSearchAPIV2Result:
    """
    `レスポンス仕様 <https://site.nicovideo.jp/search-api-docs/snapshot
    #%E3%83%AC%E3%82%B9%E3%83%9D%E3%83%B3%E3%82%B9>`_
    """

    def __init__(
        self,
        query: Dict[str, str],
        response: Union[requests.Response, List["SnapshotSearchAPIV2Result"]],
    ):
        self._fields: List[str] = (
            query["fields"].split(",") if "fields" in query else []
        )
        self._json: Optional[Dict[str, Any]] = None

        if isinstance(response, requests.Response):
            self._json: Optional[Dict[str, Any]] = response.json()
            return

        results: List["SnapshotSearchAPIV2Result"] = response

        if len(results) < 1:
            raise Exception("空のリストが渡されました")
        data = []
        meta = results[0].json()["meta"]
        for result in results:
            data.extend(result.json()["data"])
        self._json: Optional[Dict[str, Any]] = {"meta": meta, "data": data}

    def json(self) -> Dict[str, Any]:
        """
        レスポンスを Dict オブジェクトとして返す。
        """
        return self._json

    def meta(self) -> Dict[str, Any]:
        """
        リクエストのメタ情報 (HTTP Status など) を返す。
        """
        if "meta" not in self.json():
            return {}
        return self.json()["meta"]

    def status(self) -> int:
        """
        HTTP レスポンスのStatus を返す。

        複数回に分割リクエストした場合、エラー処理ができていないためここに辿り着いた時点で 200

        TODO: 分割リクエストにエラー対応を実装
        """
        return self.meta()["status"]

    def meta_id(self) -> str:
        """
        リクエストID
        """
        return self.meta()["id"]

    def total_count(self) -> int:
        """
        レスポンスに含まれる件数
        """
        return self.meta()["totalCount"]

    def data(self) -> List[Dict[str, Any]]:
        """
        レスポンスの本体データを Dict オブジェクト形式で返す。
        """
        if "data" not in self.json():
            return []
        return self.json()["data"]

    def sum_view_counter(self) -> int:
        """
        レスポンスに含まれる再生回数の合計を返す。
        """
        if FieldType.VIEW_COUNTER.value not in self._fields:
            raise Exception("フィールドに viewCounter が指定されていません")
        return sum(list(map(lambda x: x[FieldType.VIEW_COUNTER.value], self.data())))

    def avg_view_counter(self) -> int:
        """
        レスポンスに含まれる再生回数の平均を返す。
        """
        if FieldType.VIEW_COUNTER.value not in self._fields:
            raise Exception("フィールドに viewCounter が指定されていません")
        return statistics.mean(
            list(map(lambda x: x[FieldType.VIEW_COUNTER.value], self.data()))
        )

    def center_view_counter(self) -> int:
        """
        レスポンスに含まれる再生回数の中央値を返す。
        """
        if FieldType.VIEW_COUNTER.value not in self._fields:
            raise Exception("フィールドに viewCounter が指定されていません")
        return self.data()[math.floor(self.total_count() / 2)][
            FieldType.VIEW_COUNTER.value
        ]

    def text(self) -> str:
        """
        レスポンスを文字列形式で表す
        """
        return JSONEncoder().encode(self.json())



import re
from datetime import datetime
from typing import Dict, Optional, Union

from nicovideo_api_client.api.v2.limit import SnapshotSearchAPIV2Limit
from nicovideo_api_client.constants import (
    CombinedDict,
    FieldType,
    MatchDict,
    MatchValue,
    RangeDict,
    RangeValue,
)


class SnapshotSearchAPIV2SimpleFilter:
    def __init__(self, query: Dict[str, str]):
        self._query: Dict[str, str] = query

    def _cast_value(
        self, field_type: FieldType, value: Union[int, str, datetime]
    ) -> str:
        match field_type:
            case (
                FieldType.START_TIME
                | FieldType.LAST_COMMENT_TIME
                | FieldType.START_TIME
            ):
                if not isinstance(value, datetime):
                    raise TypeError(
                        f"FieldType.{field_type.value}を指定した時の型は datetime であるべきです"
                    )
                v = value.strftime("%Y-%m-%dT%H:%M:%S+09:00")
            case FieldType.CONTENT_ID:
                if isinstance(value, int):
                    v = f"sm{value}"
                elif isinstance(value, str):
                    if not re.compile(r"sm\d+").fullmatch(value):
                        raise TypeError("FieldType.CONTENT_IDはsm(数字)の形で表されるべきです")
                    v = value
                else:
                    raise TypeError("FieldType.CONTENT_IDを指定した時の型は int または str であるべきです")
            case (
                FieldType.COMMENT_COUNTER
                | FieldType.LENGTH_SECONDS
                | FieldType.LIKE_COUNTER
                | FieldType.MYLIST_COUNTER
                | FieldType.VIEW_COUNTER
            ):
                if isinstance(value, int):
                    v = value
                elif isinstance(value, str):
                    if not isinstance(int(value), int):
                        raise TypeError(f"FieldType.{field_type.value}は整数が指定されるべきです")
                    v = int(value)
                else:
                    raise TypeError(
                        f"FieldType.{field_type.value}を指定した時の型は int または str であるべきです"
                    )
            case (
                FieldType.GENRE_KEYWORD
                | FieldType.GENRE
                | FieldType.TAGS_EXACT
                | FieldType.TAGS
                | FieldType.CATEGORY_TAGS
            ):
                if not isinstance(value, str):
                    raise TypeError(
                        f"FieldType.{field_type.value}を指定した時の型は str であるべきです"
                    )
                v = value
            case (
                FieldType.TITLE
                | FieldType.DESCRIPTION
                | FieldType.USER_ID
                | FieldType.CHANNEL_ID
                | FieldType.THUMBNAIL_URL
                | FieldType.LAST_RES_BODY
            ):
                raise TypeError(
                    f"FieldType.{field_type.value}はfilterに指定できないFieldTypeです"
                )
            case _:
                raise NotImplementedError("未知のTypeが指定されました")
        return v

    def _match_filter(self, field_type: FieldType, match_value: MatchValue):
        field_count: int = 0
        for value in match_value:
            v = self._cast_value(field_type, value)

            self._query[f"filters[{field_type.value}][{field_count}]"] = v
            field_count += 1
        return self

    def _range_filter(self, field_type: FieldType, range_value: RangeValue):
        for literal, value in range_value.items():
            v = self._cast_value(field_type, value)

            self._query[f"filters[{field_type.value}][{literal}]"] = v
        return self

    def _classify(self, field_type: FieldType, value: Union[MatchValue, RangeValue]):
        if type(value) is list:
            self._match_filter(field_type, value)
        elif type(value) is dict:
            self._range_filter(field_type, value)
        else:
            raise TypeError("検索する値はListまたはDictで渡されるべきです")
        return self

    def filter(
        self,
        value: Optional[Union[MatchDict, RangeDict, CombinedDict]] = None,
        combine: bool = False,
    ) -> SnapshotSearchAPIV2Limit:
        """
        検索フィルターを指定する。

        Args:
            value(Optional[Union[MatchDict, RangeDict, CombinedDict]]):
                フィルターを指定したいフィールドの辞書
            combine(bool):
                複合検索を使用する場合にTrueを指定する。デフォルト値はFalse

        Returns:
            レスポンス要素数の指定オブジェクト

        Examples:
            #一致検索の場合: filter(MatchDict型の辞書, False)
            #範囲検索の場合: filter(RangeDict型の辞書, False)
            #複合検索の場合: filter(CombinedDict型の辞書, True)
        """
        if self._query["q"] == "" and value is None:
            raise Exception("キーワード無し検索を行う場合には必ず検索フィルタを指定してください")
        elif value is not None:
            if type(value) is not dict:
                raise TypeError("検索にはMatchDictまたはRangeDictどちらかの型を指定する必要があります")
            elif combine is True:
                field_type_list: list = []
                for k, v in value.items():
                    if k in field_type_list:
                        raise Exception("複合検索では同じFieldTypeを複数回指定することはできません")
                    field_type_list.append(k)
                    self._classify(k, v)
            else:
                for k, v in value.items():
                    self._classify(k, v)
        return SnapshotSearchAPIV2Limit(self._query)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nicovideo_api_client',
 'nicovideo_api_client.api',
 'nicovideo_api_client.api.v2']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.26.0,<3.0.0', 'urllib3>=1.26.6,<2.0.0']

setup_kwargs = {
    'name': 'nicovideo-api-client',
    'version': '0.5.0',
    'description': 'ニコニコ動画 スナップショット検索APIv2の Python クライアント',
    'long_description': '# NicoApiClient\n\n## 概要\n[ニコニコ動画 『スナップショット検索API v2』](https://site.nicovideo.jp/search-api-docs/snapshot) などの API について、仕様をなるべく意識せずに利用できるクライアントを提供する。\n\n## install\n\nPyPIリポジトリ: https://pypi.org/project/nicovideo-api-client/\n\n```shell\npip install nicovideo-api-client\n```\n\n### installed\n\n[![Downloads](https://pepy.tech/badge/nicovideo-api-client)](https://pepy.tech/project/nicovideo-api-client) [![Downloads](https://pepy.tech/badge/nicovideo-api-client/month)](https://pepy.tech/project/nicovideo-api-client) [![Downloads](https://pepy.tech/badge/nicovideo-api-client/week)](https://pepy.tech/project/nicovideo-api-client)\n\n### documentation\n\n[NicoApiClient コードドキュメント](https://javakky.github.io/NicoApiClientDocs/)\n\n## example\n\n```python\nfrom nicovideo_api_client.api.v2.snapshot_search_api_v2 import SnapshotSearchAPIV2\nfrom nicovideo_api_client.constants import FieldType\n\njson = SnapshotSearchAPIV2() \\\n    .tags_exact() \\\n    .query("VOCALOID") \\\n    .field({FieldType.TITLE, FieldType.CONTENT_ID}) \\\n    .sort(FieldType.VIEW_COUNTER) \\\n    .simple_filter().filter() \\\n    .limit(100) \\\n    .request() \\\n    .json()\n```\n\n## 利用規約\n\nhttps://site.nicovideo.jp/search-api-docs/snapshot#api%E5%88%A9%E7%94%A8%E8%A6%8F%E7%B4%84\n',
    'author': 'Javakky',
    'author_email': 'iemura.java@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Javakky/NicoApiClient',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
